Im2Col

Struct Im2Col 

pub struct Im2Col<'a, T> {
    pub image: NdTensorView<'a, T, 3>,
    pub row_offsets: RowOffsets,
    pub col_offsets: ColOffsets,
    pub n_cols: usize,
    pub n_rows: usize,
    pub max_y_offset: i32,
    pub max_x_offset: i32,
}
A matrix formed by unrolling patches of an image into columns.

Each column of the matrix corresponds to a different spatial patch of the image, and each row is a different location within the patch. The matrix can be used as the right-hand input of a matrix multiplication in order to perform a convolution.

The input image has shape [C, H, W] and is transformed into a matrix with shape [C * Kh * kW, Oh * Ow] where Kh/Kw are convolution kernel sizes and Oh/Ow are the number of patches in the Y and X directions. Given a weight matrix W of shape [M, C * Kh * kW] the matrix multiplication W @ im2col(image) produces an output of shape [M, Oh * Ow] which can be reshaped into the convolution output [M, Oh, Ow].

The matrix is virtual as it is not materialized fully in memory. Instead blocks of the matrix are materialized during computation.

Fields

image: NdTensorView<'a, T, 3>row_offsets: RowOffsets

Map of im2col row index to position within image patch (channel, kernel_y, kernel_x) pre-multiplied by corresponding stride.

The arrays may be padded to a multiple of a step size specified by the GEMM kernel. n_rows contains the actual number of rows in the virtual matrix.

col_offsets: ColOffsets

Map of im2col column index to (y, x) coordinate of top-level corner of patch in image, pre-multiplied by corresponding stride.

The arrays may be padded to a multiple of a step size specified by the GEMM kernel. n_cols contains the actual number of columns in the virtual matrix.

n_cols: usize

Number of columns in the im2col matrix.

n_rows: usize

Number of rows in the im2col matrix.

max_y_offset: i32

Maximum valid sum of row_offsets.y + col_offsets.y. Values above this correspond to the padding region.

max_x_offset: i32

Maximum valid sum of row_offsets.x + col_offsets.x. Values above this correspond to the padding region.

Implementations

Source

impl<T: Copy + Default> Im2Col<'_, T>

pub fn rows(&self) -> usize

Return the number of rows in the im2col matrix.

pub fn cols(&self) -> usize

Return the number of columns in the im2col matrix.